/***********************************************************************************************************************
 *
 * SolidBlue - open source safe data
 * Copyright (C) 2011-2012 by Tidalwave s.a.s. (http://tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://solidblue.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/solidblue-src
 *
 **********************************************************************************************************************/
package it.tidalwave.actor.netbeans.windows;

import javax.annotation.Nonnull;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import org.openide.windows.TopComponent;
import it.tidalwave.actor.spi.ActorGroupActivator;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import static org.openide.windows.TopComponent.*;

/***********************************************************************************************************************
 * 
 * @stereotype Presentation
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@Slf4j
public class ActorTopComponent<T> extends TopComponent
  { 
    @Getter
    private final T content;
    
    private final ActorGroupActivator activator;
    
    public ActorTopComponent (final @Nonnull T content,
                              final @Nonnull ActorGroupActivator activator) 
      {
        this.content = content;
        this.activator = activator;
          
        // FIXME: CenterLayout would be ok, but it renders bad the first time
        setLayout(new GridBagLayout());
        final GridBagConstraints c = new GridBagConstraints();
        c.gridwidth = 1;
        c.gridheight = 1;
        c.gridx = 0;
        c.gridy = 0;
        c.anchor = GridBagConstraints.CENTER;
        add((Component)content, c);
        putClientProperty(PROP_CLOSING_DISABLED, true);
        putClientProperty(PROP_DRAGGING_DISABLED, true);
        putClientProperty(PROP_MAXIMIZATION_DISABLED, true);
        putClientProperty(PROP_UNDOCKING_DISABLED, true);
      }

    @Override
    public void componentOpened() 
      {
        activator.activate();
      }

    @Override
    public void componentClosed() 
      {
        activator.deactivate();
      }
  }
